#!/bin/bash

# Author: Victor Correal (victor.correal@bsc.es)
# Date  : 13/10/2023 

source ../bash_utils/helpers.sh

usage() {
    info "Usage: $0 --model-dir=path_to_bundle_folder --script=name of script under SLURM/<machine>"
    info "OPTIONAL FLAGS:"
    info "[--nodes=<nodes>] [--queue=<queue>] [--MPI=<val>] [--OpenMP=<val>] [--timelimit=<time>]"
    info "[--flength=d5|h20] [--disable-io] [--outdir=<outdir>] [--io-tasks-ifs=<tasks>]"
    info "[--io-tasks-nemo=<tasks>]"
    info "[--outdir=<output_dir>] [--indir=<input_dir>]"
    info "[--force]"
}

parse_args() {
    OPTIONS=$(getopt -o "" --long model-dir:,script:,nodes:,queue:,MPI:,OpenMP:,timelimit:,disable-io,flength:,outdir:,indir:,io-tasks-nemo:,io-tasks-ifs:,force -n "$0" -- "$@")
    eval set -- "$OPTIONS"

    while true; do
        case "$1" in
            --model-dir)
                MODEL_DIR="$2"
                shift 2
            ;;
            --script)
                SCRIPT_NAME="$2"
                shift 2
            ;;
            --io-tasks-ifs)
                IO_TASKS_IFS="$2"
                shift 2
            ;;
            --io-tasks-nemo)
                IO_TASKS_NEMO="$2"
                shift 2
            ;;
            --disable-io)
                IO_ENABLED=false
                IO_TASKS_IFS=0
                IO_TASKS_NEMO=0
                shift
            ;;
            --force)
                FORCE_RUN=true
                shift
            ;;
            --flength)
                F_LENGHT="$2"
                shift 2
            ;;
            --nodes)
                NODES="$2"
                shift 2
            ;;
            --queue)
                QUEUE="$2"
                shift 2
            ;;
            --MPI)
                MPI_PER_TASK="$2"
                shift 2
            ;;
            --OpenMP)
                OMP_PER_TASK="$2"
                shift 2
            ;;
            --timelimit)
                TIME_LIMIT="$2"
                shift 2
            ;;
            --help)
                usage
                exit 0
            ;;
            --outdir)
                OUTDIR="$2"
                shift 2
            ;;
            --indir)
                INDIR="$2"
                shift 2
            ;;
            --)
                shift
                break
            ;;
            *)
                usage
                exit 1
            ;;
        esac
    done
}

print_options() {
    echo "=============================INSTALLATION==========================================="
    echo "MODEL: $MODEL_DIR"
    echo "RAPS: $RAPS_DIR"
    echo "SUBMISSION SCRIPT: $RAPS_DIR/bin/SLURM/mn5/$SCRIPT_NAME"
    echo "=============================ALLOCATION RESOURCES==================================="
    echo "QUEUE: $QUEUE, TIMELIMIT: $TIME_LIMIT"
    echo "NODE COUNT: $NODES, MPI PER NODE: $MPI_PER_NODE, OPENMP PER TASK: $OMP_PER_TASK"
    echo "================================JOB OPTIONS========================================="
    echo "RESOULTION: $RESOLUTION -- FORECAST LENGHT : $F_LENGHT -- IO_ENABLED? : $IO_ENABLED"
    echo "IO TASKS IFS: $IO_TASKS_IFS -- IO TASKS NEMO : $IO_TASKS_NEMO"
    echo "FLAGS: $OTHER_VAR"
    echo "===================================================================================="
    echo "INPUT DIR: $INDIR"  
    echo "OUT DIR: $OUTDIR"
    echo "===================================================================================="

    if [ $FORCE_RUN = false ]; then
        echo "¿FINE? (Y/N)"
        read respuesta
        case "$respuesta" in
            [Yy])
                
                ;;
            [Nn])
                echo "ABORT..."
                exit 0
                ;;
            *)
                echo "ABORT"
                exit 1
                ;;
        esac
    else
        echo "Force run ENABLE"
    fi
}

check_required_args() {
    if [ -z "${MODEL_DIR}" ] || [ -z "${SCRIPT_NAME}" ]; then
        usage
        fatal "MODEL_DIR, and SCRIPT_NAME are required."
        exit 1
    fi
}

check_dirs(){
    [ -d $MODEL_DIR ] || fatal "Error: $MODEL_DIR not found"
    
    BDIR=${MODEL_DIR}/build
    RAPS_DIR=${MODEL_DIR}/source/raps

    [ -d $BDIR ] || fatal "Error: build dir not found at $BDIR"
    [ -d $RAPS_DIR ] || fatal "Error: raps dir not found at $RAPS_DIR"

}

check_slurm_options(){
    if [ "$NODES" != "DEFAULT" ]; then
        SLURM_SYN_NODES="-N $NODES"
        echo "Overiding default for NODES: $SLURM_SYN_NODES"
    fi

    if [ "$QUEUE" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_QUEUE="-p $QUEUE"
        echo "Overiding default for QUEUE: $SLURM_SYNTAXIS_QUEUE"
    fi

    if [ "$TIME_LIMIT" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_TIME_LIMIT="--time $TIME_LIMIT"
        echo "Overiding default for TIME_LIMIT: $SLURM_SYNTAXIS_TIME_LIMIT"
    fi

    if [ "$MPI_PER_NODE" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_MPI_PER_NODE="--ntasks-per-node $MPI_PER_NODE"
        echo "Overiding default for MPI_PER_NODE: $SLURM_SYNTAXIS_MPI_PER_NODE"
    fi

    if [ "$OMP_PER_TASK" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_OMP_PER_TASK="--cpus-per-task $OMP_PER_TASK"
        echo "Overiding default for OMP_PER_TASK: $SLURM_SYNTAXIS_OMP_PER_TASK"
    fi
}
